import unittest

from video.video_track import VideoTrack
from video.video_section import VideoSection
from video.time_code import TimeCode
from undo.undo_manager import UndoManager

class Test(unittest.TestCase):

    def setUp(self):
        self.test_undo = UndoManager()
        self.track = VideoTrack("Test Track", self.test_undo)

        '''
        Adds a new video section on a track and tests that the starting and
        ending time codes are correctly recalculated. The addition is then undone and
        the track is tested to be empty again.
        
        As undoing might potentially leave the track/the action object in a
        state where redo produces unexpected results, additional undo-redo
        cycle with tests is performed in the end.
        '''


    def test_add_video_section(self):
        
        # Here are the two sections prior to adding them on the track.
        # Notice that neither starts from 0:0:0
        
        sectionA = VideoSection("Part A", TimeCode(0, 3, 0), TimeCode(0, 5, 20))

        sectionB = VideoSection("Part B", TimeCode(0, 2, 0), TimeCode(0, 4, 15))

        time1 = TimeCode(0, 2, 3)
        time2 = TimeCode(0, 2, 4)
        
        self.track.add_section(sectionA)

        # Test first that the section really was added on the track
        #
        # Basically we create an iterator to go through the sections
        # and test that the expected sections are there and that there are no extra sections
        
        section_iterator = self.track.get_section_iterator()
        self.assertEqual(sectionA, next(section_iterator),  "The newly added section - Part A - should have been on the track.")
        #After this add the track should have no more sections.
        self.assertRaises(StopIteration, next, section_iterator)
        
        
        # If all test have passed this far, we can proceed to check if the time codes
        # where changed according to the documentation. (note that this only applies to add_section)
        
        expected_start = TimeCode(0, 0, 0)
        expected_end   = TimeCode(0, 2, 20)

        self.assertEqual(str(expected_start), str(sectionA.get_start()), "Start frame timecode was wrong after add_section")
        self.assertEqual(str(expected_end), str(sectionA.get_end()), "End frame timecode was wrong after add_section")
        self.assertEqual(str(self.track), str(sectionA.get_track()), "Video track was incorrect after add_section")

        

        # Now we undo the add and check if the track is really empty(testing Undo)
        
        self.track.undo()
        
        # Get a fresh new iterator
        section_iterator = self.track.get_section_iterator()
        #After this undo the track should have been empty.
        self.assertRaises(StopIteration, next, section_iterator)
                
        # Redo should give us again the state where the section added first is on the track
        
        self.track.redo()
        
        section_iterator = self.track.get_section_iterator()
        
        self.assertEqual(sectionA, next(section_iterator),  "The first section - Part A - should have been on the track.")
        # All added sections should have been iterated over already. 
        # There shouldn't be more sections on the track.
        self.assertRaises(StopIteration, next, section_iterator)
        
        # Now we undo the redo and check if the track is once again really empty
        
        self.track.undo()
        
        # Get a fresh new iterator
        section_iterator = self.track.get_section_iterator()        
        # After this undo the track should have been empty.
        self.assertRaises(StopIteration, next, section_iterator)        
    

    def test_impossible_redo(self):
        '''
        Tests that a redo operation cannot be done when there is nothing to redo.
        '''
        self.assertFalse(self.test_undo.can_redo(), "No actions have been made so it should be impossible to redo.")
        self.assertFalse(self.track.redo(), "No actions have been made so it should be impossible to redo.")


if __name__ == "__main__":
    unittest.main()
